"""DNS Authenticator for PowerDNS."""

import logging

import zope.interface
from certbot import interfaces
from certbot import errors

from certbot.plugins import dns_common
from certbot.plugins import dns_common_lexicon

from lexicon.providers import powerdns

logger = logging.getLogger(__name__)

@zope.interface.implementer(interfaces.IAuthenticator)
@zope.interface.provider(interfaces.IPluginFactory)
class Authenticator(dns_common.DNSAuthenticator):
    """DNS Authenticator for PowerDNS DNS."""

    description = 'Obtain certificates using a DNS TXT record ' + \
                  '(if you are using PowerDNS for DNS.)'

    ttl = 60

    def __init__(self, *args, **kwargs):
        super(Authenticator, self).__init__(*args, **kwargs)
        self.credentials = None

    @classmethod
    def add_parser_arguments(cls, add):
        super(Authenticator, cls).add_parser_arguments(
            add, default_propagation_seconds=60)
        add("credentials", help="PowerDNS credentials file.")

    def more_info(self):  # pylint: disable=missing-docstring,no-self-use
        return 'This plugin configures a DNS TXT record to respond to a dns-01 challenge using ' + \
                'PowerDNS API'

    def _setup_credentials(self):
        self._configure_file('credentials',
                             'Absolute path to PowerDNS credentials file')
        dns_common.validate_file_permissions(self.conf('credentials'))
        self.credentials = self._configure_credentials(
            'credentials',
            'PowerDNS credentials file',
            {
                'api-url': 'PowerDNS-compatible API FQDN',
                'api-key': 'PowerDNS-compatible API key (X-API-Key)'
            }
        )

    def _perform(self, domain, validation_name, validation):
        self._get_powerdns_client().add_txt_record(
            domain, validation_name, validation)

    def _cleanup(self, domain, validation_name, validation):
        self._get_powerdns_client().del_txt_record(
            domain, validation_name, validation)

    def _get_powerdns_client(self):
        return _PowerDNSLexiconClient(
            self.credentials.conf('api-url'),
            self.credentials.conf('api-key'),
            self.ttl
        )


class _PowerDNSLexiconClient(dns_common_lexicon.LexiconClient):
    """
    Encapsulates all communication with the PowerDNS via Lexicon.
    """

    def __init__(self, api_url, api_key, ttl):
        super(_PowerDNSLexiconClient, self).__init__()

        config = dns_common_lexicon.build_lexicon_config('powerdns', {
            'ttl': ttl,
        }, {
            'auth_token': api_key,
            'pdns_server': api_url,
        })

        self.provider = powerdns.Provider(config)

    def _handle_http_error(self, e, domain_name):
        # Depending on server and API version, we expect certain error codes while trying
        # to guess the correct zone name. Ignore.
        if domain_name in e.response.url and e.response.status_code in [403, 404, 422]:
            return
        return super(_PowerDNSLexiconClient, self)._handle_http_error(e, domain_name)



"""Tests for certbot_dns_powerdns.dns_powerdns"""

import os
import unittest

import mock
from requests.exceptions import HTTPError

from certbot.plugins import dns_test_common
from certbot.plugins import dns_test_common_lexicon
from certbot.plugins.dns_test_common import DOMAIN

from certbot.tests import util as test_util

API_TOKEN = '00000000-0000-0000-0000-000000000000'
API_URL = 'https://127.0.0.1'


class AuthenticatorTest(test_util.TempDirTestCase,
                        dns_test_common_lexicon.BaseLexiconAuthenticatorTest):

    def setUp(self):
        super(AuthenticatorTest, self).setUp()

        from certbot_dns_powerdns.dns_powerdns import Authenticator

        path = os.path.join(self.tempdir, 'file.ini')
        dns_test_common.write(
            {"powerdns_api_url": API_URL,
             "powerdns_api_key": API_TOKEN},
            path
        )

        print("File content: ")
        # print(open(path).read())
        with open(path) as f:
            print(f.read())

        self.config = mock.MagicMock(powerdns_credentials=path,
                                     powerdns_propagation_seconds=0)  # don't wait during tests

        self.auth = Authenticator(self.config, "powerdns")

        self.mock_client = mock.MagicMock()
        # _get_powerdns_client | pylint: disable=protected-access
        self.auth._get_powerdns_client = mock.MagicMock(return_value=self.mock_client)


class PowerDnsLexiconClientTest(unittest.TestCase,
                                dns_test_common_lexicon.BaseLexiconClientTest):
    DOMAIN_NOT_FOUND = HTTPError('422 Client Error: Unprocessable Entity for url: {0}.'.format(DOMAIN))
    LOGIN_ERROR = HTTPError('401 Client Error: Unauthorized')

    def setUp(self):
        from certbot_dns_powerdns.dns_powerdns import _PowerDNSLexiconClient

        self.client = _PowerDNSLexiconClient(api_key=API_TOKEN, api_url=API_URL, ttl=0)

        self.provider_mock = mock.MagicMock()
        self.client.provider = self.provider_mock


if __name__ == "__main__":
    unittest.main()  # pragma: no cover



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Let's Encrypt PDNS plugin"""

# import inspect
#
# # https://github.com/certbot/certbot/issues/6504#issuecomment-473462138
# # https://github.com/certbot/certbot/issues/6040
# # https://github.com/certbot/certbot/issues/4351
# # https://github.com/certbot/certbot/pull/6372
# def _patch():
#     for frame_obj, filename, line, func, _, _ in inspect.stack():
#         if func == '__init__' and frame_obj.f_locals['self'].__class__.__name__ == 'PluginEntryPoint':
#             frame_obj.f_locals['self'].name = frame_obj.f_locals['entry_point'].name
#             module_name = frame_obj.f_locals['entry_point'].dist.key
#             pre_free_dist = frame_obj.f_locals['self'].PREFIX_FREE_DISTRIBUTIONS
#             if module_name not in pre_free_dist:
#                 pre_free_dist.append(module_name)
#
# _patch()


#! /usr/bin/env python
from os import path
from setuptools import setup
from setuptools import find_packages

version = "0.2.2"

with open('README.md') as f:
    long_description = f.read()

install_requires = [
    'acme>=0.31.0',
    'certbot>=0.31.0',
    'dns-lexicon>=3.2.4,<=3.5.6',
    'dnspython',
    'mock',
    'setuptools',
    'zope.interface',
    'requests'
]

here = path.abspath(path.dirname(__file__))

setup(
    name='certbot-dns-powerdns-admin',
    version=version,

    description="PowerDNS DNS Authenticator plugin for Certbot",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/pan-net-security/certbot-dns-powerdns',
    author="DT Pan-Net s.r.o",
    author_email='pannet.security@pan-net.eu',
    license='Apache License 2.0',
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Plugins',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Security',
        'Topic :: System :: Installation/Setup',
        'Topic :: System :: Networking',
        'Topic :: System :: Systems Administration',
        'Topic :: Utilities',
    ],

    packages=find_packages(),
    install_requires=install_requires,

    # extras_require={
    #     'docs': docs_extras,
    # },

    entry_points={
        'certbot.plugins': [
            'dns-powerdns-admin = certbot_dns_powerdns.dns_powerdns:Authenticator',
        ],
    },
    test_suite='certbot_dns_powerdns',
)
